@@ -4,6 +4,7 @@ import logging
 from typing import TYPE_CHECKING
 
 from pywayland.server import Listener
+
 from wlroots.util.box import Box
 from wlroots.util.edges import Edges
 
@@ -30,8 +31,8 @@ class View:
         self.x = 0.0
         self.y = 0.0
 
-        xdg_surface.map_event.add(Listener(self.xdg_toplevel_map))
-        xdg_surface.unmap_event.add(Listener(self.xdg_toplevel_unmap))
+        xdg_surface.surface.map_event.add(Listener(self.xdg_toplevel_map))
+        xdg_surface.surface.unmap_event.add(Listener(self.xdg_toplevel_unmap))
         xdg_surface.destroy_event.add(Listener(self.xdg_toplevel_destroy))
 
         toplevel = xdg_surface.toplevel
